@@ -18,19 +18,20 @@ static float jaroWinkler(const std::string_view& a, const std::string_view& b) {
     matchesB.resize(LENGTH_B);
     size_t matches = 0;
     for (size_t i = 0; i < LENGTH_A; ++i) {
-        const auto END = std::min(MATCH_DISTANCE + i + 1, LENGTH_B);
-        for (size_t j = (i > MATCH_DISTANCE ? (i - MATCH_DISTANCE) : 0); j < END; ++j) {
+        const size_t start = (i > MATCH_DISTANCE ? i - MATCH_DISTANCE : 0);
+        const size_t end   = std::min(i + MATCH_DISTANCE + 1, LENGTH_B);
+        for (size_t j = start; j < end; ++j) {
             if (matchesB[j] || a[i] != b[j])
                 continue;
-
             matchesA[i] = true;
             matchesB[j] = true;
             ++matches;
+            break;
         }
     }
 
     if (!matches)
-        return 0;
+        return 0.F;
 
     float  t = 0.F;
     size_t k = 0;
@@ -51,7 +52,7 @@ static float jaroWinkler(const std::string_view& a, const std::string_view& b) {
     return (sc<float>(matches) / LENGTH_A + sc<float>(matches) / LENGTH_B + (matches - t) / sc<float>(matches)) / 3.F;
 }
 
-constexpr const float BOOST_THRESHOLD = 0.69F;
+constexpr const float BOOST_THRESHOLD = 0.65F;
 constexpr const float FREQ_SCALE      = 0.05F;
 constexpr const float PREFIX_SCALE    = 0.1F;
 constexpr const float SUBSTR_SCALE    = 0.05F;
@@ -61,19 +62,20 @@ static float jaroWinklerFull(const std::string_view& a, const std::string_view&
     float score = jaroWinkler(a, b);
 
     // if the similarity is good enough, we can consider substr and prefix.
-    if (score > BOOST_THRESHOLD) {
+    if (score > BOOST_THRESHOLD || b.contains(a)) {
         size_t       prefixLen = 0;
-        const size_t MAXPREFIX = 4;
+        const size_t MAXPREFIX = 20;
 
         while (prefixLen < std::min({a.size(), b.size(), MAXPREFIX}) && a[prefixLen] == b[prefixLen]) {
             ++prefixLen;
         }
 
-        score += freq * FREQ_SCALE;
+        if (!prefixLen && b.contains(a))
+            score += (std::min(b.length(), sc<size_t>(4)) * SUBSTR_SCALE * (1.F - score));
+        if (prefixLen)
+            score += (sc<float>(prefixLen) * PREFIX_SCALE * (1.F - score));
 
-        if (b.contains(a))
-            return score + (std::min(b.length(), sc<size_t>(4)) * SUBSTR_SCALE);
-        return score + (sc<float>(prefixLen) * PREFIX_SCALE);
+        score += freq * FREQ_SCALE * (1.F - score);
     }
 
     return score;
@@ -100,9 +102,9 @@ static std::vector<SP<IFinderResult>> getBestResultsStable(std::vector<SScoreDat
 
     static auto getBestResult = [](std::vector<SScoreData>& data) -> typename std::vector<SScoreData>::iterator {
         typename std::vector<SScoreData>::iterator result    = data.begin();
-        float                                      bestScore = 0.F;
+        float                                      bestScore = -1.F;
         for (auto it = data.begin(); it != data.end(); ++it) {
-            if (it->score > bestScore) {
+            if (it->score > bestScore && it->score >= 0.F) {
                 bestScore = it->score;
                 result    = it;
             }
@@ -111,9 +113,9 @@ static std::vector<SP<IFinderResult>> getBestResultsStable(std::vector<SScoreDat
         return result;
     };
 
-    for (size_t i = 0; i < n; ++i) {
+    for (size_t i = 0; i < std::min(data.size(), n); ++i) {
         auto it   = getBestResult(data);
-        it->score = 0.F; // reset, don't get it again
+        it->score = -1.F; // reset, don't get it again
         resVec[i] = it->result;
     }
 
