@@ -6,6 +6,10 @@
 #include "../finders/unicode/UnicodeFinder.hpp"
 #include "../finders/math/MathFinder.hpp"
 
+#include <hyprutils/utils/ScopeGuard.hpp>
+
+using namespace Hyprutils::Utils;
+
 static WP<IFinder> finderForName(const std::string& x) {
     if (x == "desktop")
         return g_desktopFinder;
@@ -42,36 +46,9 @@ CQueryProcessor::CQueryProcessor() {
             if (m_quit)
                 break;
 
-            std::lock_guard<std::mutex> lg(m_processingMutex);
-
-            m_queryStrMutex.lock();
-
-            std::string query = m_pendingQuery;
-            m_pendingQuery    = "";
-
-            m_queryStrMutex.unlock();
-
-            WP<IFinder> FINDER;
-            bool        eat = false;
-
-            if (!m_overrideFinder) {
-                const auto [F, e] = finderForPrefix(query[0]);
-
-                if (e && query.size() == 1)
-                    continue;
-
-                FINDER = F;
-                eat    = e;
-            } else
-                FINDER = m_overrideFinder;
-
-            if (query.empty() && !m_overrideFinder)
-                continue;
-
-            auto RESULTS = FINDER ? FINDER->getResultsForQuery(eat ? query.substr(1) : query) : std::vector<SFinderResult>{};
-
-            if (g_ui && g_ui->m_backend)
-                g_ui->m_backend->addIdle([r = std::move(RESULTS)] mutable { g_ui->updateResults(std::move(r)); });
+            while (!m_quit && m_newQuery) {
+                process();
+            }
         }
     });
 }
@@ -87,6 +64,7 @@ CQueryProcessor::~CQueryProcessor() {
 void CQueryProcessor::scheduleQueryUpdate(const std::string& str) {
     m_queryStrMutex.lock();
     m_pendingQuery = str;
+    m_newQuery     = true;
     m_event        = true;
     m_queryStrMutex.unlock();
     m_threadCV.notify_all();
@@ -96,3 +74,40 @@ void CQueryProcessor::overrideQueryProvider(WP<IFinder> finder) {
     std::lock_guard<std::mutex> lg(m_processingMutex);
     m_overrideFinder = finder;
 }
+
+// Only ran on process thread
+void CQueryProcessor::process() {
+    CScopeGuard x([this] { m_newQuery = false; });
+
+    if (m_quit)
+        return;
+
+    m_queryStrMutex.lock();
+
+    std::string query = m_pendingQuery;
+    m_pendingQuery    = "";
+
+    m_queryStrMutex.unlock();
+
+    WP<IFinder> FINDER;
+    bool        eat = false;
+
+    if (!m_overrideFinder) {
+        const auto [F, e] = finderForPrefix(query[0]);
+
+        if (e && query.size() == 1)
+            return;
+
+        FINDER = F;
+        eat    = e;
+    } else
+        FINDER = m_overrideFinder;
+
+    if (query.empty() && !m_overrideFinder)
+        return;
+
+    auto RESULTS = FINDER ? FINDER->getResultsForQuery(eat ? query.substr(1) : query) : std::vector<SFinderResult>{};
+
+    if (g_ui && g_ui->m_backend)
+        g_ui->m_backend->addIdle([r = std::move(RESULTS)] mutable { g_ui->updateResults(std::move(r)); });
+}
