@@ -13,11 +13,15 @@ class CClientIPCSocket {
     bool m_connected = false;
 
     void sendOpen();
+    void sendOpenWithOptions(const std::vector<std::string>& opts);
 
   private:
     SP<Hyprwire::IClientSocket>         m_socket;
 
     SP<CCHyprlauncherCoreManagerObject> m_manager;
+    SP<CCHyprlauncherCoreInfoObject>    m_info;
+
+    bool                                m_canExit = false;
 
     std::string                         m_socketPath = "";
 };
